/*******************************************
* Basic motor test
* Copy AFMotor library and RobotMotor libraries to Arduino library folder
* Navigate to RobotMotor: 
*   for 4WD, copy RobotMotor4wd/RobotMotor.cpp into RobotMotor folder
*   for 2WD, copy RobotMotor2wd/RobotMotor.cpp into RobotMotor fold
* 
* Strategy:
* - keep moving straigt until an object is detected
* - robot is sweeping 60->90->120 degrees back and then back to 60 degrees
* - if an object is to close, ...
* 
* Tony Sena,        Apr 26, 2017
* Steiger-Escobar,  May  2, 2017   Simplified algorithm
* 
* 
********************************************/

// include motor libraries
#include <AFMotor.h>     // adafruit motor shield library
#include <RobotMotor.h>  // 2wd or 4wd motor library
#include <Servo.h>

//----------------------------------------------------------------------------
// VARIABLE DECLARATIONS
//----------------------------------------------------------------------------

// threshold: object is too close, need to change direction (inches)
const int threshold = 20;

Servo myservo;              // create servo object to control a servo

// servo position we will use
const int AHEAD = 90;
const int RIGHT = 60;
const int LEFT = 120;

const int servoPin =  9;    // pin used for servo motor on motor shield
const int pingPin  = 10;    // pin used for the ultrasonic sensor's output on motor shield

int moveSpeed   = 100;      // the speed to run (as a % of maximum speed: [0,100]) 
int pauseMs     = 1500;     // time in ms for each movement
int servoDelay  = 1000;

int currentSpeed = 0;       // current speed of the motor

void setup()
{
  Serial.begin(9600);       // initialize serial communication to display data
  startCar(); 

  myservo.attach(servoPin); // attaches the servo on pin 9 to the servo object
  myservo.write(AHEAD);     // tell servo to point ahead
  delay(servoDelay);        // waits 15ms for the servo to reach the position
}

void loop()
{ 
  if ( distanceFromObject() > threshold ){
    //
    // YOUR CODE GOES HERE!!!
    // Hint: Your collision avoidance strategies (week-7).
    //
  } else {
    //
    // YOUR CODE GOES HERE!!!
    // Hint: Your collision avoidance strategies (week-7).
    //
  }  
}

void moveForward()
{
  Serial.println("forward");
  //
  // YOUR CODE GOES HERE!!!
  // Hint: week-6 Lab-2, MotorTest2wd sketchbook and MotorTest2wd-iCREAT sketchbook. 
}

void moveBackward()
{
  Serial.println("backward");
  //
  // YOUR CODE GOES HERE!!!
  // Hint: week-6 Lab-2, MotorTest2wd sketchbook and MotorTest2wd-iCREAT sketchbook. 
}

void turnLeft()
{
  Serial.println("Turn left");  
  //
  // YOUR CODE GOES HERE!!!
  // Hint: week-6 Lab-2, MotorTest2wd sketchbook and MotorTest2wd-iCREAT sketchbook. 
}

void turnRight()
{
  Serial.println("Turn right");  
  //
  // YOUR CODE GOES HERE!!!
  // Hint: week-6 Lab-2, MotorTest2wd sketchbook and MotorTest2wd-iCREAT sketchbook. 
}

void rotateLeft()
{
  Serial.println("Rotating left");  
  //
  // YOUR CODE GOES HERE!!!  
  // Hint: week-6 Lab-2, MotorTest2wd sketchbook and MotorTest2wd-iCREAT sketchbook. 
}

void rotateRight()
{
  Serial.println("Rotating right");  
  //
  // YOUR CODE GOES HERE!!!
  // Hint: week-6 Lab-2, MotorTest2wd sketchbook and MotorTest2wd-iCREAT sketchbook. 
}

void startCar()
{
  Serial.println("Start Motors");
  //
  // YOUR CODE GOES HERE!!!
  // Hint: week-6 Lab-2, MotorTest2wd sketchbook and MotorTest2wd-iCREAT sketchbook. 
}
  
void stopCar()
{
  Serial.println("Stop Motors");
  //
  // YOUR CODE GOES HERE!!!
  // Hint: week-6 Lab-2, MotorTest2wd sketchbook and MotorTest2wd-iCREAT sketchbook. 
}

void stopAndWait(int waitTime)
{
  Serial.println("Stop and Wait");
  motorStop(MOTOR_LEFT);
  motorStop(MOTOR_RIGHT);
  delay(waitTime);
}

//-------------------------------------------------------------------//
//--------------  Funtions needed for Ultrasonic Sensor  ------------//
//-------------------------------------------------------------------//

// This functions returns the distance from the object in inches
long distanceFromObject() {
  // establish variables for duration of the ping, and distance result in inches and centimeters
  long duration, inches, cm;

  // The PING))) is triggered by a HIGH pulse of 2 or more microseconds.
  // Give a short LOW pulse beforehand to ensure a clean HIGH pulse:
  pinMode(pingPin, OUTPUT);
  digitalWrite(pingPin, LOW);
  delayMicroseconds(2);

  digitalWrite(pingPin, HIGH);
  delayMicroseconds(5);
  digitalWrite(pingPin, LOW);

  // The same pin is used to read the signal from the PING))): a HIGH
  // pulse whose duration is the time (in microseconds) from the sending
  // of the ping to the reception of its echo off of an object.
  pinMode(pingPin, INPUT);
  duration = pulseIn(pingPin, HIGH);

  // use a function to convert the time into a distance
  inches = microsecondsToInches(duration);
  cm     = microsecondsToCentimeters(duration);

  Serial.print(inches);
  Serial.print("in, ");
  Serial.print(cm);
  Serial.print("cm");
  Serial.println();

  delay(100);

  // return the distance from the object in inches
  return inches;
}

// According to Parallax's datasheet for the PING))), there are
// 73.746 microseconds per inch (i.e. sound travels at 1130 feet per
// second).  This gives the distance travelled by the ping, outbound
// and return, so we divide by 2 to get the distance of the obstacle.
// See: http://www.parallax.com/dl/docs/prod/acc/28015-PING-v1.3.pdf
long microsecondsToInches(long microseconds) 
{
  return (microseconds / 74 / 2);
}

// The speed of sound is 340 m/s or 29 microseconds per centimeter.
// The ping travels out and back, so to find the distance of the
// object we take half of the distance travelled.
long microsecondsToCentimeters(long microseconds) 
{
  return (microseconds / 29 / 2);
}

//-------------------------------------------------------------------//
//------------------------- TEST FUNTIONS ---------------------------//
//-------------------------------------------------------------------//

void iCREATtest()
{
  // rotate left, rotate right, move forward, and move backwards
  rotateLeft();
  delay(pauseMs); // run for 2 seconds
  stopAndWait(pauseMs);
  rotateRight();
  delay(pauseMs); // run for 2 seconds
  stopAndWait(pauseMs);
  moveForward();
  delay(pauseMs); // run for 2 seconds
  stopAndWait(pauseMs); 
  moveBackward();
  delay(pauseMs); // run for 2 seconds
  stopAndWait(pauseMs); 

  // move forward from 0 to 100% (max speed), wait for 5sec, move backwards from max speed (100%) down to 0
  moveForwardWithSpeed(); 
}

void moveForwardWithSpeed()
{
  // start motors
  motorBegin(MOTOR_LEFT);       
  motorBegin(MOTOR_RIGHT); 

  // move forward from 0 to 100% (max speed) 
  Serial.println("forward with speed 0 to 100%");  
  for (currentSpeed = 0; currentSpeed <= 100; currentSpeed += 1) {
    motorForward(MOTOR_LEFT,  currentSpeed);
    motorForward(MOTOR_RIGHT, currentSpeed);
    delay(100); // run for 100ms
  }

  delay(5000); // run for 5 seconds 5sec

  // move backwards from max speed (100%) down to 0 
  Serial.println("backward with speed 100% to 0"); 
  for (currentSpeed = 100; currentSpeed > 0; currentSpeed -= 1) {
    motorReverse(MOTOR_LEFT,  currentSpeed);
    motorReverse(MOTOR_RIGHT, currentSpeed);
    delay(100); // run for 100ms
  }

  stopAndWait(pauseMs);
}


